package com.ys.product.ysmq.front.strategy;

import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import com.ys.product.ysmq.front.exceptions.YsmqConsumerMessageException;
import com.ys.product.ysmq.front.log.Log;
import com.ys.product.ysmq.front.log.LogFactory;
import com.ys.product.ysmq.front.msg.AbstractConsumerMessage.ConsumerCallBack;

/**
 * <p>
 * 一个本地队列的示例
 * </p>
 * 
 * @author jiangyuechao 2019年9月4日 上午11:53:22
 * @version V1.0
 * @modificationHistory=========================逻辑或功能性重大变更记录
 * @modify by user: {修改人} 2019年9月4日
 * @modify by reason:{方法名}:{原因}
 */
public class LocalQueueMessageErrorStrategy implements HandleMessageErrorStrategy {

    private static Log log = LogFactory.getLog(LocalQueueMessageErrorStrategy.class);

    private BlockingQueue<List<Object>> queue = new LinkedBlockingQueue<>(2000);

    public LocalQueueMessageErrorStrategy(ConsumerCallBack consumerCallBack) {
        this.consumerCallBack = consumerCallBack;
    }

    @Override
    public void handleMessageError(List<Object> data) {
        try {
            queue.put(data);
        } catch (InterruptedException e) {
            log.error(String.format("handleMessageError出错,data:%s.", data), e);
        }
    }

    protected void consumerMessage(List<Object> data) {
        try {
            while (true) {
                List<Object> take = queue.take();
                consumerCall(take);
            }
        } catch (Exception e) {
            log.error(String.format("handleMessageError出错,data:%s.", data), e);
        }
    }

    ConsumerCallBack consumerCallBack;

    public void setConsumerCallBack(ConsumerCallBack consumerCallBack) {
        this.consumerCallBack = consumerCallBack;
    }

    public void consumerCall(List<Object> msg) {
        if (consumerCallBack == null) {
            throw new YsmqConsumerMessageException("回调函数不可为空,请设置回调函数!");
        }
        this.consumerCallBack.consumerCall(msg);
    }

}
