package com.ys.product.ysmq.front.utils;

import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.TimeUnit;

import com.ys.product.ysmq.front.exceptions.YsmqHttpException;
import com.ys.product.ysmq.front.log.Log;
import com.ys.product.ysmq.front.log.LogFactory;
import com.ys.product.ysmq.front.model.Const;

import okhttp3.ConnectionPool;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

public class OkHttpHelper {

    private static Log logger = LogFactory.getLog(OkHttpHelper.class);

    static ConnectionPool pool = new ConnectionPool(6, 10, TimeUnit.MINUTES);

    static OkHttpClient okHttpClient;

    public static OkHttpClient closeableOKHTTP() {

        OkHttpClient client = new OkHttpClient.Builder() //
                .connectTimeout(3, TimeUnit.MINUTES) //
                .followRedirects(true) //
                .readTimeout(3, TimeUnit.MINUTES) //
                .retryOnConnectionFailure(false) //
                .writeTimeout(3, TimeUnit.MINUTES).connectionPool(pool) //
                .build();
        return client;
    }

    public static void destroy() {

        if (okHttpClient != null) {
            ConnectionPool connectionPool = okHttpClient.connectionPool();
            connectionPool.evictAll();
            ExecutorService executorService = okHttpClient.dispatcher().executorService();
            executorService.shutdown();
            try {
                executorService.awaitTermination(3, TimeUnit.MINUTES);
            } catch (InterruptedException e) {
            }
        }
    }

    private static String accessToken;

    /**
     * <p>
     * 获取token令牌
     * </p>
     * 
     * @param url
     * @param appKey
     * @param appSecret
     * @return token
     * @throws IOException
     */
    public static String getAccessToken(String url, String appKey, String appSecret) throws IOException {
        // FormBody的类型就是application/x-www-form-urlencoded
        FormBody formBody = new FormBody.Builder().add(Const.APPKEY, appKey).add(Const.APPSECRET, appSecret).build();
        OkHttpClient client = OkHttpHelper.getOkHttpClient();
        Request request = new Request.Builder().url(url).post(formBody).build();
        try (Response response = client.newCall(request).execute()) {
            if (response.code() == 200) {
                accessToken = ResponseParseUtil.parseAccessToken(response.body().string());
            } else {
                // 打印错误
                logger.error("http没有正常返回,code:{},body:{}", response.code(), response.body().string());
                throw new YsmqHttpException("code:" + response.code() + ", body:" + response.body().string());
            }
            return accessToken;
        }
    }

    public static OkHttpClient getOkHttpClient() {
        if (okHttpClient == null) {
            okHttpClient = closeableOKHTTP();
        }
        return okHttpClient;
    }

}
